classdef FDAProcess < handle
   
    % Vrifier qu'il n'y ait aucun lment li aux interfaces graphiques :
    % ok vrifier
    
    %% Definition des proprietes de la classe dataProcess
    properties
        currentDirApp       char        % Chemin d'accs de l'app
        currentDataFile     char        % nom du fichier de donnees courant
        mydata              struct      % Ensemble de donnes courant
        mydata_ini          struct      % Ensemble de donnes initial avant tout prtraitement
        mydataPretreated    struct      % Ensemble de donnes avec pretraitement stat choisi par l'utilisateur
        pretreatment        char        % Pretraitement applique
        mypca               struct      % Structure SAISIR contenant les resultats de la PCA
        myfda               struct      % Structure SAISIR contenant les resultats de la FDA

        % Proprit de fluorescence
        sizeEm      double              % nombre de longueurs d'onde en emission = dimension 1 de la matrice EEM en fluorescence
        sizeEx      double              % nombre de longueurs d'onde en excitation = dimension 2 de la matrice EEM en fluorescence
        startEm     double              % valeur (en nm) de dpart en emission de la matrice EEM
        startEx     double              % valeur (en nm) de dpart en excitation de la matrice EEM
        
        endEm       double              % valeur finale (en nm) en emission de la matrice EEM       
        endEx       double              % valeur finale (en nm) en excitation de la matrice EEM
        stepEm      double              % pas de progression (en nm) en emission 
        stepEx      double              % pas de progression (en nm) en excitation 
%         
%         bddSignalsFluo  cell            % tableau excel ou quivalent des fluorophores connus dans la matrice tudiee
        deviceType      char            % information sur l'instrument ayant produit les donnees
%         
%         % Proprits de Chromatographie LC et GC
%         
%         % Proprits de RMN
%         eretic          double
%         
%         % Proprit gnrale dfinissant l'instrument
        datatype char               % Nature des donnees acquises "Fluo", "GC", "MS" etc...
    end
 
    %% Methodes = fonctions de la classe dataProcess, qui vont travailler sur les proprietes 
    
    methods
        function obj = FDAProcess(currentDirApp, currentDataFile)
            
            obj.currentDirApp = currentDirApp; 
            obj.currentDataFile = currentDataFile;
        end
        
        function loadData(obj, currentDirApp, currentDataFile)
            
            myfileName = fullfile(currentDirApp, currentDataFile);
            dataset = load(myfileName);
            obj.mydata = dataset.mydata;
            obj.mydata_ini = dataset.mydata;
        end
        
        function loadData_metaData(obj)
            
            % on renseigne les proprits en fonction de la nature des
            % donnes : Fluo, GC etc...
            
            switch obj.mydata.datatype
                case "Fluo"
                    if isfield(obj.mydata,'datatype')
                        obj.datatype = obj.mydata.datatype;
                    else
                        obj.datatype ='champ "datatype" non prsent';
                    end
                    if isfield(obj.mydata,'sizeEm')
                        obj.datatype = obj.mydata.datatype;
                    else
                        obj.datatype ='champ "sizeEm" non prsent';
                    end
                    obj.sizeEm = obj.mydata.sizeEm;
                    obj.sizeEx = obj.mydata.sizeEx;
                    obj.startEm = obj.mydata.startEm;
                    obj.startEx = obj.mydata.startEx;
                    obj.endEm = obj.mydata.endEm;
                    obj.endEx = obj.mydata.endEx;
                    obj.stepEm = obj.mydata.stepEm;
                    obj.stepEx = obj.mydata.stepEx;
                    obj.deviceType = obj.mydata.deviceType;
                    
                    
                case "GC"
                    
                case "MS"
                    
                case "IRRaman"
                    
                case "Physicochimie"
                    
                case "IRMS"
                    
            end
            
            % Proprits communes
            obj.datatype = obj.mydata.datatype;
        end
        
        function makeDataPretreatments(obj, selectedPT)
            % fonction de prtraitement statistique
            % selectedPT (type double) = selection du preptraitement a realiser 
            
            obj.mydata_ini = obj.mydata;       % set de donnes total avant slection de variables ou pretraitement
            obj.mydataPretreated = obj.mydata_ini; % on travaille sur la variable mydataSet_ini_pretreated
            
           try
                switch selectedPT
                    case 1 % Standard Normal Variate alone
                        obj.mydataPretreated = snv(obj.mydata); % prtraitement du set de donnes total initial
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = 'SNV';
                        
                    case 2 % First derivative alone
                        h = waitbar(0,'Please wait, derivating in progress...','name','ChromStat pretreatment procedure');
                        mydataSet = saisir_derivative(obj.mydata, 3, 201, 1);
                        obj.mydataPretreated = mydataSet;
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        close(h)
                        obj.pretreatment = '1st derivative';
                        
                    case 3 % First derivative + snv
                        mydata_FirstDeriv = saisir_derivative(obj.mydata, 3, 201, 1);
                        obj.mydataPretreated = snv(mydata_FirstDeriv);
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = '1st derivative + snv';
                        
                    case 4 % Pareto scaling alone
                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_matrix_pareto = (obj.mydata.d - meanX)./sqrt(std(obj.mydata.d));
                        red_matrix_pareto(isnan(red_matrix_pareto))= 0;
                        red_matrix_pareto = matrix2saisir(red_matrix_pareto);
                        obj.mydataPretreated = red_matrix_pareto;
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = 'Pareto scaling';
                        
                    case 5 % First derivative + snv + Pareto scaling
                        
                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_matrix_pareto = (obj.mydata.d - meanX)./sqrt(std(obj.mydata.d));
                        red_matrix_pareto(isnan(red_matrix_pareto))= 0;
                        red_matrix_pareto = matrix2saisir(red_matrix_pareto);
                        mydataSet = red_matrix_pareto;
                        
                        mydataSet = snv(mydataSet);
                        mydata_FirstDeriv = saisir_derivative(mydataSet, 3, 201, 1);
                        
                        obj.mydataPretreated = mydata_FirstDeriv;
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = '1st derivative + snv + Pareto scaling';
                        
                    case 6 % Range scaling alone
                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_matrix_RangeScale = (obj.mydata.d - meanX)./(max(obj.mydata.d) - min(obj.mydata.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);
                        
                        obj.mydataPretreated = red_matrix_RangeScale;
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = 'Range scaling';
                        
                    case 7 % Pareto + Range scaling
                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_matrix_pareto = (obj.mydata.d - meanX)./sqrt(std(obj.mydata.d));
                        red_matrix_pareto(isnan(red_matrix_pareto))= 0;
                        red_matrix_pareto = matrix2saisir(red_matrix_pareto);
                        mydataSet = red_matrix_pareto;
                        
                        red_matrix_RangeScale = (mydataSet.d - meanX)./(max(mydataSet.d) - min(mydataSet.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);
                        
                        obj.mydataPretreated = red_matrix_RangeScale;
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = 'Pareto scaling + Range scaling';
                        
                    case 8 % First derivative + snv + Range scaling
                        
                        mydataSet = snv(obj.mydata);
                        meanX = mean(mydataSet.d);
                        meanX = repmat(meanX, size(mydataSet.d, 1),1);
                        red_matrix_RangeScale = (mydataSet.d - meanX)./(max(mydataSet.d) - min(mydataSet.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);
                        mydata_snv_RangeScale_FirstDeriv = saisir_derivative(red_matrix_RangeScale, 3, 201, 1);
                        
                        obj.mydataPretreated = mydata_snv_RangeScale_FirstDeriv;
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = '1st derivative + snv + Range scaling';
                        
                    case 9 % 1st Deriv + Range scaling
                        mydata_FirstDeriv = saisir_derivative(obj.mydata, 3, 201, 1);
                        meanX = mean(mydata_FirstDeriv.d);
                        meanX = repmat(meanX, size(mydata_FirstDeriv.d, 1),1);
                        red_matrix_RangeScale = (mydata_FirstDeriv.d - meanX)./(max(mydata_FirstDeriv.d) - min(mydata_FirstDeriv.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);
                        
                        obj.mydataPretreated = red_matrix_RangeScale;
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = '1st derivative + Range scaling';
                        
                    case 10 % snv + Range scaling + Pareto
                        mydataSet = snv(obj.mydata);
                        meanX = mean(mydataSet.d);
                        meanX = repmat(meanX, size(mydataSet.d, 1),1);
                        red_matrix_pareto = (mydataSet.d - meanX)./sqrt(std(mydataSet.d));
                        red_matrix_pareto(isnan(red_matrix_pareto))= 0;
                        red_matrix_pareto = matrix2saisir(red_matrix_pareto);
                        
                        red_matrix_RangeScale = (red_matrix_pareto.d - meanX)./(max(red_matrix_pareto.d) - min(red_matrix_pareto.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);
                        
                        obj.mydataPretreated = red_matrix_RangeScale;
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = 'snv + Range scaling + Pareto';
                        
                    case 11 % Level scaling alone
                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_levelScaling = (obj.mydata.d - meanX)./meanX;
                        red_levelScaling(isnan(red_levelScaling))= 0;
                        red_levelScaling = matrix2saisir(red_levelScaling);
                        
                        obj.mydataPretreated = red_levelScaling;
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = 'Level scaling';
                        
                    otherwise
                        obj.mydataPretreated = obj.mydata;      % aucun pretraitement
                        obj.pretreatment = 'none';
                end
           catch err
               msgbox({'La mthode "makeDataPretreatments" attend un paramtre numrique li au choix du prtraitement Stat.',...
                   err.message},'Erreur sur une methode', 'error');
           end
        end
        
        function makeFDA(obj, optPOS, maxComposantes, NbParmiMax, plotBoolean)
            
            if not(isempty(obj.mypca))
                % calcul du modle FDA sur les composantes principales sur le dataset courant
                mygroups = create_group1(obj.mydata, optPOS(1), optPOS(2));
               
                obj.myfda = fda1(obj.mypca, mygroups, maxComposantes, NbParmiMax);                  % FDA sur le set de donnes total avec pretraitement
                obj.myfda.comment = 'FDA sur set de donnes complet';
            end
            if strcmp(plotBoolean,'factorPlot')
                                figure('color',[1 1 1]);
                gscatter(obj.myfda.datafactor.d(:,1), obj.myfda.datafactor.d(:,2), mygroups.d);
                legend(mygroups.g.i,'interpreter','none');

                                figure('color',[1 1 1]);
                ellipse_map(obj.myfda.datafactor, 1, 2, mygroups, 0);
                symbol_map(obj.myfda.datafactor, 1, 2, 1, 5); hold on;
                legend(mygroups.g.i,'interpreter','none');

                
            end
        end  
    end
end
















