classdef PCAProcess < dynamicprops

    % PCAProcess : Classe process permettant de raliser une ACP avec
    % plusieurs prtraitement au choix sur un ensemble de donnes au format
    % SAISIR

    %% Definition des proprietes de la classe dataProcess
    properties
        currentDirApp       char        % Chemin d'accs de l'app
        currentDataFile     char        % nom du fichier de donnees courant
        mydata              struct      % Ensemble de donnes courant
        mydata_ini          struct      % Ensemble de donnes initial avant tout prtraitement
        mydataPretreated    struct      % Ensemble de donnes avec pretraitement stat choisi par l'utilisateur
        pretreatment        char        % Pretraitement applique
        mypca               struct      % Structure SAISIR contenant les resultats de l'ACP

        %         % Proprit de fluorescence

        %     sizeEm      double              % nombre de longueurs d'onde en emission = dimension 1 de la matrice EEM en fluorescence
        %     sizeEx      double              % nombre de longueurs d'onde en excitation = dimension 2 de la matrice EEM en fluorescence
        %     startEm     double              % valeur (en nm) de dpart en emission de la matrice EEM
        %     startEx     double              % valeur (en nm) de dpart en excitation de la matrice EEM
        %
        %     endEm       double              % valeur finale (en nm) en emission de la matrice EEM
        %     endEx       double              % valeur finale (en nm) en excitation de la matrice EEM
        %     stepEm      double              % pas de progression (en nm) en emission
        %     stepEx      double              % pas de progression (en nm) en excitation
        %
        %     bddSignalsFluo  cell            % tableau excel ou quivalent des fluorophores connus dans la matrice tudiee

        %     deviceType      char            % information sur l'instrument ayant produit les donnees

        % Proprits de Chromatographie LC et GC

        % Proprits de RMN
        %         eretic          double

        % Proprit gnrale dfinissant l'instrument
        %         datatype char               % Nature des donnees acquises "Fluo", "GC", "MS" etc...


    end

    %% Methodes = fonctions de la classe dataProcess, qui vont travailler sur les proprietes

    methods
        function loadData(obj, pathname, fileName)

            myfileName = fullfile(pathname,fileName);
            dataset = load(myfileName);
            obj.mydata = dataset.mydata;
            obj.mydata_ini = obj.mydata;       % set de donnees total avant slection de variables ou pretraitement

        end

        function obj = PCAProcess(param1, param2)
        %
        % Constructeur de la Classe PCAProcess
        % Deux faons de l'appeler :
        % 
        % Mthode 1 : vos donnes sont dans un fichier .MAT qui contient une variable nomme "mydata" au format SAISIR (.d; .i; .v)
        %       Appel :
        %               options  = {"Nom-du_fichier.mat",type_de_pretraitement}; 
        %               objetPCA = PCAProcess(currentDir, options)
        %
        % Mthode 2 : vos donnes sont dans le workspace de Matlab dans la variable mydata au format SAISIR
        %       Appel :
        %               objPCA = PCAProcess(mydata, type_de_pretraitement)
        %               
        % INPUTS
        %   options     :   variable de type cell contenant 2 cellules, la 1ere contient le nom du fichier de donnees .MAT, la 2nde
        %                   contient le type de prtraitement (type entier) souhait pour les donnes  choisir parmi la liste :
        %                   (1)  SNV
        %                   (2)  First derivative alone
        %                   (3)  First derivative + snv
        %                   (4)  Pareto scaling alone
        %                   (5)  First derivative + snv + Pareto scaling
        %                   (6)  Range scaling alone
        %                   (7)  Pareto + Range scaling
        %                   (8)  First derivative + snv + Range scaling
        %                   (9)  1st Deriv + Range scaling
        %                   (10) snv + Range scaling + Pareto
        %                   (11) Level scaling alone
        %
        %   currentDir  :   variable de type char, chemin d'accs au fichier de donnes .MAT
        %   mydata      :   jeu de donnes au format SAISIR
        %
        % OUTPUT
        %   objetPCA    : objet de type PCAProcess contenant 
        %                 - currentDataFile	    nom du fichier de donnees courant
        %                 - currentDirApp	    Chemin d'accs de l'app
        %                 - mydata	            Ensemble de donnes courant
        %                 - mydataPretreated	    Ensemble de donnes avec pretraitement stat choisi par l'utilisateur
        %                 - mydata_ini	            Ensemble de donnes initial avant tout prtraitement
        %                 - mypca	            Structure SAISIR contenant les resultats de l'ACP
        %                 - pretreatment	    Pretraitement applique
        %
        % Comment afficher les rsultats de l'ACP ?
        %
        %   Exemple pour construire un graphe : figure('color', [1 1 1]); colored_map1(objetPCA.mypca.score, 1, 2, 5, 5);
        %   
        %   Bien sur, il faut adapter les valeurs que vous passez en
        %   paramtres et notamment les 2 derniers qui sont les positions
        %   des caractres  prendre en compte pour dfinir les groupes
        %
        %   Exemple pour rcuprer le jeu de donnes : mydata = objetPCA.mydata;
        %   
        %
            if nargin==2
                if issaisir(param1)
                    obj.currentDirApp = pwd;
                    obj.currentDataFile = "variable in workspace";
                    obj.pretreatment = 'none';
                    obj.mydata_ini = param1;
                    obj.mydata = obj.mydata_ini;
                    disp("'mydata' variable loaded from workspace")

                    makeDataPretreatments(obj, param2)
                    makePCA(obj)
                else
                    if exist(param2{1},"file")
                        loadData(obj, param1, param2{1})
                        disp("'mydata' variable loaded from datafile")
                        obj.currentDirApp = param1;
                        obj.currentDataFile = param2{1};
                        obj.pretreatment = param2{2};

                        makeDataPretreatments(obj, param2{2})
                        makePCA(obj)
                    end
                end
            end
             if nargin<2
                    obj.currentDirApp = pwd;
                    obj.currentDataFile = "none datafile: load datafile to complete the PCAProcess class";
                    obj.pretreatment = 'none';
             end

        end


        function loadData_metaData(obj)

            % on ajoute des mta data aux proprits de la structure SAISIR
            % si ncessaire en chargeant ces metadata  partir d'un fichier
            % complmentaire dfini dans l'en-tete d'appel de l'App

            switch obj.mydata.datatype
                case "Fluo"
                    obj.sizeEm = obj.mydata.sizeEm;
                    obj.sizeEx = obj.mydata.sizeEx;
                    obj.startEm = obj.mydata.startEm;
                    obj.startEx = obj.mydata.startEx;
                    obj.endEm = obj.mydata.endEm;
                    obj.endEx = obj.mydata.endEx;
                    obj.stepEm = obj.mydata.stepEm;
                    obj.stepEx = obj.mydata.stepEx;
                    obj.deviceType = obj.mydata.deviceType;

                case "GC"

                case "HRMS"

                case {"Raman",'IR'}

                case "Physicochimie"

                case "IRMS"

                case {"NMR","RMN"}

            end

        end
        function makeDataPretreatments(obj, selectedPT)
            % function makeDataPretreatments(obj, selectedPT)
            %
            % fonction de prtraitement statistique
            % selectedPT (type double) = selection du preptraitement a realiser
            % INPUT : selectedPT
            % (1)  SNV
            % (2)  First derivative alone
            % (3)  First derivative + snv
            % (4)  Pareto scaling alone
            % (5)  First derivative + snv + Pareto scaling
            % (6)  Range scaling alone
            % (7)  Pareto + Range scaling
            % (8)  First derivative + snv + Range scaling
            % (9)  1st Deriv + Range scaling
            % (10) snv + Range scaling + Pareto
            % (11) Level scaling alone

            obj.mydataPretreated = obj.mydata_ini; % on travaille sur la variable mydataSet_ini_pretreated

            % application d'une correction avant le pretraitement
            if isfield(obj.mydata,"datatype")
            else
                obj.mydata.datatype = "not defined";
            end
            switch obj.mydata.datatype
                case {'RMN','NMR'}
                    if ~strcmp(obj.pretreatment, 'Eretic')
                        [n, ~] = size(obj.mydataPretreated.d);
                        obj.mydataPretreated.d = obj.mydataPretreated.d ./ obj.mydata.eretic(1:n);
                        obj.pretreatment = 'Eretic';
                    else
                        disp('La correction par le coefficient ERETIC a dj t applique. L''ensemble de donnes est conserv ainsi.');
                    end

                case {'LC-MS','GC-MS'}
                case {'Fluorescence','FFFS', 'Fluo'}
                case {'GC', 'Fast-GC', 'Heracles'}
                case {'Infrarouge', 'IR', 'Raman'}
                case {'Images','Imagerie','images','hyperspectral'}
                case {'Physicochimie','PC','Physicochemical'}

                case 'IRMS'
                    obj.pretreatment = 'Aucun prtraitement';

                otherwise
                    obj.mydata.datatype = 'type de donnes  dfinir';
                    obj.pretreatment = 'Aucun prtraitement';
            end
           


            try
                switch selectedPT
                    case 0 % aucun pretraitement
                        obj.mydataPretreated = obj.mydata; % auncun pretraitement
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = 'Aucun';

                    case 1 % Standard Normal Variate alone
                        obj.mydataPretreated = snv(obj.mydata); % prtraitement du set de donnes total initial
                        obj.mydataPretreated.i = obj.mydata_ini.i;
                        obj.pretreatment = 'SNV';

                    case 2 % First derivative alone
                        h = waitbar(0,'Please wait, derivating in progress...','name','ChromStat pretreatment procedure');
                        mydataSet = saisir_derivative(obj.mydata, 3, 41, 1);
                        obj.mydataPretreated = mydataSet;
                        obj.mydataPretreated.i = obj.mydata.i;
                        close(h)
                        obj.pretreatment = '1st derivative';

                    case 3 % First derivative + snv
                        mydata_FirstDeriv = saisir_derivative(obj.mydata, 3, 201, 1);
                        obj.mydataPretreated = snv(mydata_FirstDeriv);
                        obj.mydataPretreated.i = obj.mydata.i;
                        obj.pretreatment = '1st derivative + snv';

                    case 4 % Pareto scaling alone
                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_matrix_pareto = (obj.mydata.d - meanX)./sqrt(std(obj.mydata.d));
                        red_matrix_pareto(isnan(red_matrix_pareto))= 0;
                        red_matrix_pareto = matrix2saisir(red_matrix_pareto);
                        obj.mydataPretreated = red_matrix_pareto;
                        obj.mydataPretreated.i = obj.mydata.i;
                        obj.pretreatment = 'Pareto scaling';

                    case 5 % First derivative + snv + Pareto scaling

                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_matrix_pareto = (obj.mydata.d - meanX)./sqrt(std(obj.mydata.d));
                        red_matrix_pareto(isnan(red_matrix_pareto))= 0;
                        red_matrix_pareto = matrix2saisir(red_matrix_pareto);
                        mydataSet = red_matrix_pareto;

                        mydataSet = snv(mydataSet);
                        mydata_FirstDeriv = saisir_derivative(mydataSet, 3, 201, 1);

                        obj.mydataPretreated = mydata_FirstDeriv;
                        obj.mydataPretreated.i = obj.mydata.i;
                        obj.pretreatment = '1st derivative + snv + Pareto scaling';

                    case 6 % Range scaling alone
                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_matrix_RangeScale = (obj.mydata.d - meanX)./(max(obj.mydata.d) - min(obj.mydata.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);

                        obj.mydataPretreated = red_matrix_RangeScale;
                        obj.mydataPretreated.i = obj.mydata.i;
                        obj.pretreatment = 'Range scaling';

                    case 7 % Pareto + Range scaling
                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_matrix_pareto = (obj.mydata.d - meanX)./sqrt(std(obj.mydata.d));
                        red_matrix_pareto(isnan(red_matrix_pareto))= 0;
                        red_matrix_pareto = matrix2saisir(red_matrix_pareto);
                        mydataSet = red_matrix_pareto;

                        red_matrix_RangeScale = (mydataSet.d - meanX)./(max(mydataSet.d) - min(mydataSet.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);

                        obj.mydataPretreated = red_matrix_RangeScale;
                        obj.mydataPretreated.i = obj.mydata.i;
                        obj.pretreatment = 'Pareto scaling + Range scaling';

                    case 8 % First derivative + snv + Range scaling

                        mydataSet = snv(obj.mydata);
                        meanX = mean(mydataSet.d);
                        meanX = repmat(meanX, size(mydataSet.d, 1),1);
                        red_matrix_RangeScale = (mydataSet.d - meanX)./(max(mydataSet.d) - min(mydataSet.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);
                        mydata_snv_RangeScale_FirstDeriv = saisir_derivative(red_matrix_RangeScale, 3, 201, 1);

                        obj.mydataPretreated = mydata_snv_RangeScale_FirstDeriv;
                        obj.mydataPretreated.i = obj.mydata.i;
                        obj.pretreatment = '1st derivative + snv + Range scaling';

                    case 9 % 1st Deriv + Range scaling
                        mydata_FirstDeriv = saisir_derivative(obj.mydata, 3, 201, 1);
                        meanX = mean(mydata_FirstDeriv.d);
                        meanX = repmat(meanX, size(mydata_FirstDeriv.d, 1),1);
                        red_matrix_RangeScale = (mydata_FirstDeriv.d - meanX)./(max(mydata_FirstDeriv.d) - min(mydata_FirstDeriv.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);

                        obj.mydataPretreated = red_matrix_RangeScale;
                        obj.mydataPretreated.i = obj.mydata.i;
                        obj.pretreatment = '1st derivative + Range scaling';

                    case 10 % snv + Range scaling + Pareto
                        mydataSet = snv(obj.mydata);
                        meanX = mean(mydataSet.d);
                        meanX = repmat(meanX, size(mydataSet.d, 1),1);
                        red_matrix_pareto = (mydataSet.d - meanX)./sqrt(std(mydataSet.d));
                        red_matrix_pareto(isnan(red_matrix_pareto))= 0;
                        red_matrix_pareto = matrix2saisir(red_matrix_pareto);

                        red_matrix_RangeScale = (red_matrix_pareto.d - meanX)./(max(red_matrix_pareto.d) - min(red_matrix_pareto.d));
                        red_matrix_RangeScale(isnan(red_matrix_RangeScale))= 0;
                        red_matrix_RangeScale = matrix2saisir(red_matrix_RangeScale);

                        obj.mydataPretreated = red_matrix_RangeScale;
                        obj.mydataPretreated.i = obj.mydata.i;
                        obj.pretreatment = 'snv + Range scaling + Pareto';

                    case 11 % Level scaling alone
                        meanX = mean(obj.mydata.d);
                        meanX = repmat(meanX, size(obj.mydata.d, 1),1);
                        red_levelScaling = (obj.mydata.d - meanX)./meanX;
                        red_levelScaling(isnan(red_levelScaling))= 0;
                        red_levelScaling = matrix2saisir(red_levelScaling);

                        obj.mydataPretreated = red_levelScaling;
                        obj.mydataPretreated.i = obj.mydata.i;
                        obj.pretreatment = 'Level scaling';

                    otherwise
                        obj.mydataPretreated = obj.mydata;      % aucun pretraitement
                        obj.pretreatment = 'none';
                end
            catch err
                msgbox({'La mthode "makeDataPretreatments" attend un paramtre numrique li au choix du prtraitement Stat.',...
                    err.message},'Erreur sur une methode', 'error');
            end
        end

        function makePCA(obj)
            % calcul des composantes principales sur le dataset courant
            obj.mypca = pca(obj.mydataPretreated);                  % ACP sur le set de donnes total avec pretraitement
            obj.mypca.comment = 'PCA sur set de donnes complet';

        end
    end
end
















