function plstype=basic_pls(X,y,ndim)
%function plstype=basic_pls(X,y,ndim);
%Input arguments:
%================
%X: SAISIR matrix of predictive variables (n x p) 
%y: SAISIR vector of variables to be predicted (n x 1) 
%ndim: maximum number of dimension asked
%
%Output arguments:
%=================
%plstype with fields:
% BETA: regression coefficients for the ndim models (p x ndim)
% T: PLS scores (n x ndim) 
% P:PLS loadings such as X = TP + residuals (ndim x p) 
% Q: Q value such as y = TQ + residuals (ndim x 1) 
% BETA0: intercepts of the ndim models (ndim x 1) 
% PREDY: predicted y values up to ndim dimensions (n x ndim)
% RMSEC: Root mean square error of predicttion up to ndim dimensions (1 x ndim)
% r2: determination coefficient between observed and predicted values (1 x ndim) 
%
%source : adaptation of the SCILAB FACT program "ikpls" as developped by
%Jean Claude Boulet (INRA Montpellier France and Jean-Micher Roger (IRSTEA CEMAGREF Montpelliet France)
%
%Dominique says " Ridge regression always surpasses PLS"
%
%Note : if you need to project supplementary observations in the PLS space 
% use basic_pls1, which is much slower .
%see also: basic_pls1,basic_pls2, pcr, spcr, ridge_regression, ridge_regression1, lr1,applypls


[n,p]=size(X.d);
[X Xmean] = center(X);
[y1 ymean]=center(y);
[plstype.BETA.d plstype.T.d plstype.P.d plstype.Q.d dof]=fact_ikpls(X.d,y1.d,ndim);
%%res.BETA0.d=meany-meanx*res.BETA.d;
% 'ymean'
% size(ymean.d)
% 'xmean'
% size(Xmean.d)
% 'BETA'
% plstype.BETA.d
plstype.BETA0.d=ymean.d-Xmean.d*plstype.BETA.d;
plstype.BETA0.i='constant';
plstype.BETA0.v=addcode(num2str(dof),y.v);
plstype.BETA.i=X.v;
plstype.BETA.v=plstype.BETA0.v;
plstype.T.i=X.i;
plstype.T.v=num2str(dof);
plstype.P.i=num2str(dof);
plstype.P.v=X.v;
plstype.Q.i=plstype.BETA.v;
plstype.Q.v='Coeff of regression using T';
%%plstype.PREDY.d=X.d*plstype.BETA.d+ ones(n,1)*plstype.BETA0.d;
%%warning ! as X.d is here centered, the complete regression formula does
%%not includes the BETA0. Only the mean of y appears here.  
plstype.PREDY.d=X.d*plstype.BETA.d+ ymean.d;
plstype.PREDY.i=y.i;
plstype.PREDY.v=plstype.BETA0.v;
delta=(y.d*ones(1,ndim)-plstype.PREDY.d);
plstype.RMSEC.d=sqrt(sum(delta.*delta)/n);
plstype.RMSEC.i='Root mean square error of calibration';
plstype.RMSEC.v=plstype.BETA.v;
plstype.r2=cormap(y,plstype.PREDY);
plstype.r2.d=plstype.r2.d.*plstype.r2.d;

%% local function 
function [b,T,P,Q,dof] = fact_ikpls(x,y,nf)
%adapted from function ikpls of SCILAB fact package
%distributed under the CeCILL-C license
%copyright INRA-IRSTEA 2013

  W=[]; 
  R=[];
  P=[];
  Q=[];
  T=[];
  XY=x'*y;
  for i=1:nf;
    w=XY;
    w=w/sqrt(w'*w);
    r=w;
    for j=1:i-1;
      r=r-(P(:,j)'*w)*R(:,j);
    end
    t=x*r;
    tt=t'*t;
    p=x'*t/tt;
    q=(r'*XY)'/tt;
    XY=XY-(p*q')*tt;
    T=[T t];
    P=[P p];
    Q=[Q q];
    R=[R r];
 end
  
  b=cumsum(R*diag(Q'),2);
   

  dof=[1:1:nf]';
P=P';%% this is an option: P is such that X=TP;
Q=Q';

