function modeleKovats = calculerModeleKovats(vectorTR, vectorIx, typeModelKovats,options)
% calcule un modèle Kovats pour la détermmination des indices de Kovats en Fast-GC
%
% modeleKovats = calculerModeleKovats(vectorTR, vectorIx, typeModelKovats,options)
%
try
    %     vectorTR = app.KovatsIndices_UITable.Data{:,2};
    %     vectorIx = app.KovatsIndices_UITable.Data{:,3};
    %
    %     choix = strfind(app.axb_DropDown.Items,app.axb_DropDown.Value);
    %     positionSelectionDropDown = find(cellfun(@isempty, choix)==0);

    switch typeModelKovats
        case "linear"
            [~, ~, b, pente,~, ~] = computeKovatsIxEquation(vectorIx, vectorTR, 1);
            T = table(vectorTR, vectorIx);
            T.Properties.VariableNames = {'Actual Retention Time (RT)','Kovats indices (Ix)'};
            mdl = fitlm(T,"linear");

            if nargin==4 && strcmp(options,"display")
                %                 figure; plot(gca, vectorTR, vectorIx, 'bo', "Marker","o", "MarkerFaceColor",[0 0 1]);
                %                 hlin = lsline(gca); set(hlin, "color", "r", "linestyle","-.");
                %                 xlabel("Actual retention times (s)"); ylabel("Theoretical Kovats indices")

                %axb_Label.Text = ['$' num2str(pente) 'x +  ' num2str(b) '$'];
                figure("Color",[1 1 1]);
                plot(gca, mdl)
                annotation(gcf,'textbox',...
                    [0.614285714285714 0.228571428571429 0.267857142857143 0.144238095238096],...
                    'String',{['RMSE : ', num2str(mdl.RMSE)], ['Adjusted Rsquared : ' num2str(mdl.Rsquared.Adjusted)],'model : linear',...
                    ['Ix = ', num2str(mdl.Coefficients{2,1}) ' * RT + ' num2str(mdl.Coefficients{1,1})]},...
                    'LineStyle','none',...
                    'FitBoxToText','off');


            end
            modeleKovats.linear.a = pente;
            modeleKovats.linear.b = b;
            modeleKovats.mdl = mdl;

        case "polynomial"
            [~, ~, ~, ~, polynom, ~] = computeKovatsIxEquation(vectorIx, vectorTR, 2);

            T = table(vectorTR, vectorIx);
            T.Properties.VariableNames = {'Actual Retention Time (RT)','Kovats indices (Ix)'};
            mdl = fitlm(T,"quadratic");

            %axb_Label.Text = ['$' num2str(polynom.p1) 'x^2 +  ' num2str(polynom.p2) 'x + ' num2str(polynom.p3) '$'];

            if nargin==4 && strcmp(options,"display")
                %                 plot(gca, vectorTR, f,'bo', "Marker","o", "MarkerFaceColor",[0 0 1]);
                %                 hold(gca, "on");
                %                 plot(gca,vectorTR,f);
                plot(mdl);
                annotation(gcf,'textbox',...
                    [0.614285714285714 0.228571428571429 0.267857142857143 0.144238095238096],...
                    'String',{['RMSE : ', num2str(mdl.RMSE)], ['Adjusted Rsquared : ' num2str(mdl.Rsquared.Adjusted)],'model : quadratic',...
                    ['Ix = ', num2str(mdl.Coefficients{2,1}) ' * RT + ' num2str(mdl.Coefficients{1,1})]},...
                    'LineStyle','none',...
                    'FitBoxToText','off');

                hold(gca, "off");
            end
            modeleKovats.polynomial.a = polynom.p1;
            modeleKovats.polynomial.b = polynom.p2;
            modeleKovats.polynomial.c = polynom.p3;
            modeleKovats.mdl = mdl;
    end

catch erreur
    msgbox(erreur.message)
end
end
