classdef calibValidPartition2<dynamicprops
    % Classe qui renferme toutes les méthodes de sélection des noms de fichiers.

    properties
        mydata  % jeu de donnees au format SAISIR (.d, .i, .v)
        mygroups
        calibSet = {};
        validSet = {};
        classesEchantillons = {}; % Liste des index correspondants aux différentes classes d'échantillons trouvés. Chaque entrée est une classe d'échantillons avec ses index respectifs.
        existingClassesNames = {}; % Noms arbitraires de classes trouvés selon les index de lettres fournis lors de la création de l'objet.
        nbReplicatsPerGroup = 0;
    end

    methods
        function obj = calibValidPartition2(mydata, firstLetter, LastLetter, valP, calP, firstLetterClass, lastLetterClass)
            obj.mydata = mydata;
            groupsResults = makeGroups(obj, firstLetter, LastLetter, firstLetterClass, lastLetterClass);
            selectSamples(obj, valP, calP, groupsResults)
        end

        function groupsResults = makeGroups(obj, firstLetter, LastLetter, firstLetterClass, lastLetterClass)
            obj.mygroups = create_group1(obj.mydata, firstLetter, LastLetter);
            groupsNames = cellstr(obj.mygroups.g.i);

            groups = {};
            obj.nbReplicatsPerGroup = 0;
            for i=1:size(groupsNames, 1)
                index = find(obj.mygroups.d==i);
                groups{i,1} = selectrow(obj.mydata, index);
                obj.nbReplicatsPerGroup = obj.nbReplicatsPerGroup+size(groups{i,1}.d, 1);
                % Nom du groupe - Pris sur le premier élément des noms ; Par exemple '1_F'
                name = groups{i,1}.i(1,:);
                % Détermination du nom du groupe selon les index fournis
                grpName = name(firstLetterClass:lastLetterClass);
                % Index du groupe testé
                indexG = find(strcmp(obj.existingClassesNames, grpName), 1);
                % Vérification si le groupe n'est pas déjà connu
                if not(isempty(indexG))
                    % Si le groupe est déjà connu, on ajoute l'index (donc
                    % i) à la table correspondante à son groupe. (indexG) 
                    obj.classesEchantillons{indexG}{end+1, 1} = i;
                else
                    % Si le groupe n'existe pas encore, on le créé (avec grpName) puis on
                    % insère son premier élément (donc i)
                    obj.existingClassesNames{end+1, 1} = grpName;
                    obj.classesEchantillons{end+1, 1} = {i};
                end
            end
            %obj.nbReplicatsPerGroup = size(obj.mydata.i,1)/size(obj.classesEchantillons, 1);
            groupsResults = groups;
        end

        function selectSamples(obj, valP, calP, groupsResults)
            % Effectue la sélection des groupes selon la proportion indiquée.
            % valP : Pourcentage de échantillons à sélectionner pour la partie calul,
            % calP : le reste.

            if nargin < 3
                valP = 1/3;
                calP = 2/3;
            end
            nbGroups = size(groupsResults, 1);
            disp(strcat("Sibling Groups found : ", string(nbGroups)))
            disp(strcat("Master groups found : ", string(size(obj.classesEchantillons, 1))))
            if (valP + calP) ~= 1
                error('Invalid percentages, sum of valP and calP should be equal to 1.');
            end

            % Détermination du nombre de groupe d'échantillons à placer dans le groupe de calibration.
            %tocal = round(nbGroups * calP);

            % Initialisation des groupes de calibration et de validation
            obj.calibSet = {};
            %calibSetNames = {};
            obj.validSet = {};

            %% Détermination du nombre de groupes de réplicats à piocher dans chaque classe.

            % Explication du calcul : on divise le nb total de groupes par
            % le nombre de classes d'échantillons on multiplie ensuite par
            % le pourcentage de groupes de réplicats à mettre dans le set
            % de calibration.
            disp('-----------------------------------')
            disp(strcat('Nombre moyen de réplicats par échantillon :', string(obj.nbReplicatsPerGroup/nbGroups)))
            obj.nbReplicatsPerGroup = obj.nbReplicatsPerGroup/size(obj.classesEchantillons, 1);
            for e=1:size(obj.existingClassesNames, 1)
                disp('-----------------------------------')
                disp(strcat('Stats pour les échantillons du groupe : ',obj.existingClassesNames{e}))
                perClass = round(size(obj.classesEchantillons{e}, 1)*calP);
                disp(strcat('Calibration - nombre de groupes de réplicats :', string(perClass)))
                disp(strcat('Validation - nombre de groupes de réplicats :', string(size(obj.classesEchantillons{e}, 1)-perClass)))
            end
            aux = [];
            %% IL EST PROBABLEMENT POSSIBLE D'OPTIMISER CE CODE MAIS LA
            %% FONCTION CELLFUN NE SEMBLE PAS ADAPTEE CAR ELLE RENVOIE
            %% UN VECTEUR AVEC LA MÊME DIMENSION QUE CELUI DONNE EN ENTREE
            %% OR, CE N'EST PAS CE QU'ON CHERCHE À FAIRE, ON VEUT UNE SEULE
            %% STRUCTURE MYDATA POUR LES DONNEES DE CALIBRATION ET DE VALIDATION
            %% LES BOUCLES SONT DONC NECESSAIRES

            for i = 1:size(obj.existingClassesNames, 1)
                % Récupération des index correspondants au groupe
                % actuellement traité.
                currentClass = obj.classesEchantillons{i};
                nbClass = size(currentClass, 1)*calP;
                for y = 1:nbClass
                    
                    % Tirage d'un nombre pour sélectionner le groupe.
                    selectedGroupIndex = randi(size(currentClass, 1));

                    % Récupération de l'index réel du groupe
                    group2add = currentClass{selectedGroupIndex};

                    % Ajout de l'ensemble échantillon et ses réplicats à la
                    % liste calibSet et supression de l'index de la liste
                    % de ceux disponibles.
                    obj.calibSet = appendrow1(obj.calibSet, groupsResults{group2add});
                    currentClass(selectedGroupIndex) = [];
                    
                end
                % Une nouvelle boucle est nécessaire pour récupérer les
                % index restants pour les mettre dans la table de
                % validation car on ne peut pas modifier la table
                % groupResults sinon les index contenus dans currentClass
                % ne seront plus valides.
                for z = 1:size(currentClass, 1)
                    % Récupération de l'index réél dans la table
                    % groupsResults
                    group2add = currentClass{z};

                    % Ajout de la structure mydata à celle déjà existante
                    % avec la fonction appendrow1
                    aux = appendrow1(aux, groupsResults{group2add});
                end
            end
            obj.validSet = aux;

            %%
            %aux = [];
            % Boucle sur le nombre de groupes à mettre en calibration.
%             for i = 1:tocal
%                 % Tirage au sort d'un entier entre 1 et le nombre de groupes pas déjà tirés au sort.
%                 echNb = randi(nbGroups - i);
% 
%                 % Ajout de l'ensemble échantillon et ses réplicats à la liste calGrp
%                 obj.calibSet = appendrow1(obj.calibSet, groupsResults{echNb});
%                 groupsResults(echNb) = [];
% 
%             end
% 
%             % Les groupes restants sont donc ceux sélectionnés pour la catégorie de validation.
%             obj.validSet = groupsResults;
%            for i=1:size(groupsResults)
%                 aux = appendrow1(aux, obj.validSet{i});
%            end
%            obj.validSet = aux;
        end
    end

end

