function res=covariance_select(X,y,nvar,standardize)
%covariance_select  - selection of predictive variables from covariance
%function res=covariance_select(X,Y,nvar,standardize);
%
%Input arguments
%===============
%X  :Saisir matrix of predictive variable (n x p)
%y  :Saisir vector of variables to be predicted (n x k)
%nvar: Number of variables introduced in the model
%standardize(optional): if true, the data are standardized before covsel application ; false(default) : data not standardized
%Output argument
%===============
%res with fields:
%-res.rank : indices of the selected variables (up to nvar variables)
%-res.cumulated_variance : percentage of the variance reconstructible from
%the selected variables in X . First column: cumulated variance of X, second
%column : cumulated variance in Y.
%res.reducedX: X reduced to selected variables
%
%According to :
%J.M Roger, B. Palagos, D.Bertrand; E.Fernadez-Ahumuda. CovSel: Slection
%de variables dans le cas hautement multivari et d'talonnage
%multi-rponses. Application  la spectromtrie IR. Congress "Chimiomtrie
%2009, Paris 30 november-1st December 2009"
%AgroParisTech, 48-51.
%
%%Typical example:
%%Asking for 10 variables
% res=covariance_select(X,y,10)
% res =
%                   rank: [1x1 struct]
%     cumulated_variance: [1x1 struct]
%
%see also: stepwise_regression
if(nargin<4)
    [rank.d,cumulated_variance.d] = covsel(X.d,y.d,nvar,0);%% not standardized
else
    [rank.d,cumulated_variance.d] = covsel(X.d,y.d,nvar,standardize);
end
rank.d=rank.d';
rank.v='rank of the introduced variable';
rank.i=X.v(rank.d,:);
cumulated_variance.i=num2str((1:nvar)');
cumulated_variance.v=char({'varX';'varY'});
aux=selectcol(X,rank.d);
%res1=multiple_regression(aux,y);

res.rank=rank;
res.cumulated_variance=cumulated_variance;
res.reducedX=selectcol(X,rank.d);



