function [num,crbvar] = covsel(x,y,lv,standardize)
%covsel       - Selection of variable from covariance maximisation
%  [num,crbvar] = covsel(x,y,lv)
%
% Input arguments
%================
%x: Matlab matrix nxp
%y : Matlab matrix n x q
%standardize (optional): if false (default) data not standardized, true: data standardized
%Output arguments
%================
%num    : number (rank) of selected variables
%crbvar : evolution des variances cumules de X (1ere colonne) et de Y (2eme)

crbvar=zeros(lv,2);

[nx,px] = size(x);
[ny,py] = size(y);

if nx ~= ny
    error('Le nombre d''individus ne correspond pas')
end

lv = min( lv, rank(x) );
xstand=0;
if(nargin>3)
    xstand=standardize;
end
if(xstand==0)
    z=auto(x);
else
    z=mncn(x);
end
t=auto(y);

% variance (inertie) totale
vartot=[sum(sum(z.*z)) sum(t.*t)];
for i=1:lv
    xdisp(i,' among', lv);
    [cm,j] = max(z'*t.*z'*t);
    v = z(:,j);
    z = ( eye(nx)-(v*v')./(v'*v) )*z;
    crbvar(i,1)=(vartot(1)-sum(sum(z.*z)))/vartot(1);
    t = ( eye(nx)-(v*v')./(v'*v) )*t;
    crbvar(i,2) = (vartot(2)-trace(t'*t))/vartot(2);
    num(i)=j;
end

%% local function auto ==================================================
function [ax,mx,stdx] = auto(x)
%AUTO Autoscales matrix to mean zero unit variance
%  Autoscales a matrix (x) and returns the resulting matrix (ax)
%  with mean-zero unit variance columns, a vector of means (mx)
%  and a vector of standard deviations (stdx) used in the scaling.
%


[m,n] = size(x);
mx    = mean(x);
stdx  = std(x);
ax    = (x-mx(ones(m,1),:))./stdx(ones(m,1),:);


%% local function mncn
function [mcx,mx] = mncn(x)
%MNCN Mean center scales matrix to mean zero.
%  Mean centers matrix (x), returning a matrix with
%  mean zero columns (mcx) and the vector of means
%  (mx) used in the scaling.

[m,n] = size(x);
mx    = mean(x);
mcx   = (x-mx(ones(m,1),:));
