function spectre = importNIRcsvfile(filename, dataLines)
%IMPORTFILE Import data from a text file
%  SAMPLEF4R3 = IMPORTFILE(FILENAME) reads data from text file FILENAME
%  for the default selection.  Returns the numeric data.
%
%  SAMPLEF4R3 = IMPORTFILE(FILE, DATALINES) reads data for the specified
%  row interval(s) of text file FILENAME. Specify DATALINES as a
%  positive scalar integer or a N-by-2 array of positive scalar integers
%  for dis-contiguous row intervals.
%
%  Example:
%  sampleF4R3 = importfile("/Users/lartic/Downloads/Projets_collaboratifs_du_departement/Projet_Fromages_Julien/IR/NIR/sample_F4-R3.csv", [3, Inf]);
%
%  See also READTABLE.
%
% Auto-generated by MATLAB on 05-Apr-2024 18:02:07

%% Input handling

% If dataLines is not specified, define defaults
if nargin < 2
    dataLines = [3, Inf];
end

%% Set up the Import Options and import the data
opts = delimitedTextImportOptions("NumVariables", 2);

% Specify range and delimiter
opts.DataLines = dataLines;
opts.Delimiter = [",", ";"];

% Specify column names and types
opts.VariableNames = ["Wavelength", "Absorbance"];
opts.VariableTypes = ["double", "double"];

% Specify file level properties
opts.ExtraColumnsRule = "ignore";
opts.EmptyLineRule = "read";

% Import the data
spectre = readtable(filename, opts);

%% Convert to output type
spectre = table2array(spectre);
end