function spectrum = loadIRCSVFile(filePath, fileName)

%% Import data from text file
% Script for importing data from the transmitted text file: CSV
%
% return table object containing Intensity as 1st column and wavelength as
% 2nd column
% 

%% Set up the Import Options and import the data
opts = delimitedTextImportOptions("NumVariables", 2);

% Specify range and delimiter
opts.DataLines = [1, Inf];
opts.Delimiter = ";";

% Specify column names and types
opts.VariableNames = ["Wavelength", "Intensity"];
opts.VariableTypes = ["double", "double"];

% Specify file level properties
opts.ExtraColumnsRule = "ignore";
opts.EmptyLineRule = "read";

% Specify variable properties
opts = setvaropts(opts, ["Wavelength", "Intensity"], "TrimNonNumeric", true);
opts = setvaropts(opts, ["Wavelength", "Intensity"], "DecimalSeparator", ",");
opts = setvaropts(opts, ["Wavelength", "Intensity"], "ThousandsSeparator", ".");

% Import the data
spectrum = readtable(fullfile(filePath, fileName), opts);


%% Clear temporary variables
clear opts