function [icaRes] = saisir_ica(mydata, NBIc, Options)
%% Perfom ICA on SAISIR matrix
% [icaRes] = saisir_ica(mydata, NBIc, Options)
% 
%
% Inputs :
% mydata: SAISIR Matrix
% NBIc: number of ICs to be used in the model
% Options: model calculation options: "Wide", "Tall", "Normal"
% Options.Method = 'Tall', 'Wide', 'Kernel', 'ComDim', 'Normal'
% Options.Partitions = number of partition if X is tall or wide
% Options.Centred = Matrix centred by column
%
% Output : 
% icaRes: SAISIR structure specific to ICA
%
% Author : C.B.Y. Cordella, UMR914 Inra PNCA, Laboratoire de Chimie Analytique
% 2017
% Use : jade_DNR, IC_signs from D. N Rutledge & D. Jouan-Rimbaud Bouveresse


[n,p] = size(mydata.d);

[Sig,Sco] = jade_DNR(mydata.d, NBIc, Options);
[SignalICA, ScoresICA] = IC_signs( Sig, Sco , Options); 

icaRes.scores.d = ScoresICA;
icaRes.scores.i = mydata.i;
icaRes.scores.v = [1:n]';
icaRes.score.d = ScoresICA;
icaRes.score.i = mydata.i;
icaRes.score.v = [1:n]';

icaRes.signals.d = SignalICA;
icaRes.signals.i = mydata.i;
icaRes.signals.v = mydata.v;

icaRes.Date = date;
icaRes.Author = 'Christophe B.Y. Cordella - Univ. Laval, Qubec, Canada';


